import { Module } from '@nestjs/common';
import { FisVerificationController } from './fis-verification.controller';
import { FisVerificationService } from './fis-verification.service';
import { ClientsModule, Transport } from '@nestjs/microservices';
import configuration from '../../../config/configurations';
import 'dotenv/config'
import { ConfigModule, ConfigService } from '@nestjs/config';
@Module({
  imports: [
    ConfigModule.forRoot({
      isGlobal: true,
      load: [configuration]
    }),
    ClientsModule.registerAsync([
      {
        name: `SAMPLEAPP_SERVICE`,
        imports: [ConfigModule],
        useFactory: async (configService: ConfigService) => ({
          transport: Transport.TCP,
          options: {
            host: configService.get<string>(`sampleApp.host`) as string,
            port: configService.get<number>(`sampleApp.tcpPort`) as number,
            retryAttempts: 5,
            retryDelay: 5000
          }
        }),
        inject: [ConfigService]
      },
      {
        name: `JAVA_AFIS`,
        imports: [ConfigModule],
        useFactory: async (configService: ConfigService) => ({
          transport: Transport.TCP,
          options: {
            host: configService.get<string>(`afis.host`) as string,
            port: configService.get<number>(`afis.tcpPort`) as number,
            retryAttempts: 5,
            retryDelay: 5000
          }
        }),
        inject: [ConfigService]
      },
    ])
  ],
  controllers: [FisVerificationController],
  providers: [FisVerificationService],
})
export class FisVerificationModule { }
