import { NestFactory } from '@nestjs/core';
import { SampleAppModule } from './sample-app.module';
import { MicroserviceOptions, Transport } from '@nestjs/microservices';
import { ConfigService } from '@nestjs/config';
import 'dotenv/config'

async function bootstrap() {
  // This example contains a hybrid application (HTTP + TCP + Socket)
  const app = await NestFactory.create(SampleAppModule);
  const config: ConfigService = app.get(ConfigService)

  const host: string = config.get<string>(`sampleApp.host`) as string
  const restPort: number = config.get<number>(`sampleApp.restPort`) as number
  const tcpPort: number = config.get<number>(`sampleApp.tcpPort`) as number

  app.connectMicroservice<MicroserviceOptions>({
    transport: Transport.TCP,
    options: {
      host: host,
      port: tcpPort,
      retryAttempts: 5,
      retryDelay: 3000
    },
  });

  await app.startAllMicroservices();
  await app.listen(restPort, host);
  console.log(`REST API is running on: ${await app.getUrl()} and TCP is running on ${host}:${tcpPort}`);
}
bootstrap();
