import { Module } from '@nestjs/common';
import { SampleAppController } from './sample-app.controller';
import { SampleAppService } from './sample-app.service';
import { ClientsModule, Transport } from '@nestjs/microservices';
import { SampleEventsGateway } from './sample.gateway';
import { ConfigModule, ConfigService } from '@nestjs/config';
import configuration from '../../../config/configurations';
import 'dotenv/config'
@Module({
  imports: [
    ConfigModule.forRoot({
      isGlobal: true,
      load: [configuration]
    }),
    ClientsModule.registerAsync([
      {
        name: `FINGERPRINT_SERVICE`,
        imports: [ConfigModule],
        useFactory: async (configService: ConfigService) => ({
          transport: Transport.TCP,
          options: {
            host: configService.get<string>(`fingerprint.host`) as string,
            port: configService.get<number>(`fingerprint.tcpPort`) as number
          }
        }),
        inject: [ConfigService]
      },
      {
        name: `VERIFICATION_SERVICE`,
        imports: [ConfigModule],
        useFactory: async (configService: ConfigService) => ({
          transport: Transport.TCP,
          options: {
            host: configService.get<string>(`verification.host`) as string,
            port: configService.get<number>(`verification.tcpPort`) as number
          }
        }),
        inject: [ConfigService]
      }
    ]),
    /* Sync register version -- Use at your own risk.*/
    // ClientsModule.register([
    //   {
    //     name: `FINGERPRINT_SERVICE`,
    //     transport: Transport.TCP,
    //     options: {
    //       host: ConfigService.get<string>(`sampleApp.host`) as string,
    //       port: process.env.FINGERPRINT_TCP_PORT as unknown as number
    //     }
    //   },
    // ])
  ],
  controllers: [SampleAppController],
  providers: [SampleAppService, SampleEventsGateway],
})
export class SampleAppModule { }
